classdef DG1022 < hgsetget
    % DG1022 Minimal Implementation of a controller class for Rigol
    % Arbitrary Waveform Generator DG1022
    %
    % Design goal is to use only basic Visa commands "fprintf" and "fgets"
    % for maximum compatibility
    %
    % RIGOL_DG1022
    % Implementierung einer Klasse fuer Arbitrary Function Generator
    %
    % DG1022 Properties:
    %  this class has no user properties
    %
    %% Features
    % - Of couse, all features not supported by this class (see
    %   limitations) can be controlled manually with SCPI commands. Every
    %   awg class has "fprintf" and "query" commands implemented for this
    %   purpose.
    %
    %% Limitations
    % - Only tested with Rigol DG1022
    % - Mod, Burst and Sweep not supported (yet)
    % - Groups: Pulse, AM, FM, PM, FSK, Sweep, Trigger, Burst
    %   not implemented yet due to no requirements
    % - Memory group: not implemented (and will not be implemented)
    % - Display-Group: not implemented (and will not be implemented)
    % - Coupling-Group: not implemented
    % - Counter-Group: not implemented
    % - Use of external Trigger: not implemented (yet)
    % - Apply-Group: Not implemented
    % - Voltage-Group: No test if voltages are sensible implemented
    % - Output-Load-Setting: Not implemented (yet)
    % - Trigger-Setting: Not implemented (yet)
    %
    %
    %% Remarks
    % Implementation of an *.mdd-driver makes no sense, since the "query"-
    % command of the "test & measurement tool" (tmtool) usually fails
    % (whereas "write" and "read" instructions work- seems to be a timeout
    % issue).
    %
    %
    %
    % 2.11.2014 Nicolas (Check www.dl1dow.de for a more recent version)

    %% Implementation
    % The following is grouped by SCPI-commands
    
    
    
    
    
    properties
        Interface;
    end
    
    
    methods
        % Constructor
        function obj = DG1022(varargin)
            
        end
        
        % Methods using VISA Interface
        connect(obj);
        disconnect(obj);
        [varargout] = query(obj,varargin);
        Out = instrhwinfo(~,varargin);   % Need no ressources
    end
    
    
    %% IEEE 488.2 (full)
    % *IDN?           -> getInstrumentModel
    % *RST            -> devicereset
    methods
        out = getInstrumentModel(obj);
        devicereset(obj);
    end
    
    
    %% Function-Group ("Commands for channels", full)
    % FUNC            -> setFunc
    % FUNC?           -> getFunc
    % FUNC:USER       -> setFunc
    % FUNC:USER?      -> getFunc
    % FUNC:SQU:DCYC   -> setDutycycle
    % FUNC:SQU:DCYC?  -> getDutycycle
    % FUNC:RAMP:SYMM  -> setSymmetry
    % FUNC:RAMP:SYMM? -> getSymmetry
    % FUNC:CH2        -> setFunc
    % FUNC:CH2?       -> getFunc
    % FUNC:USER:CH2   -> setFunc
    % FUNC:USER:CH2?  -> getFunc
    % FUNC:SQU:DCYC:CH2 -> setDutycycle
    % FUNC:SQU:DCYC?:CH2 -> getDutycycle
    % FUNC:RAMP:SYMM:CH2 -> setSymmetry
    % FUNC:RAMP:SYMM?:CH2 -> getSymmetry
    %
    % function group is identical for both Channels
    methods
        setFunc(Obj,chNo,fcnName);
        out = getFunc(Obj,chNo);
        setDutycycle(Obj,chNo,value);
        out = getDutycycle(Obj,chNo)
        setSymmetry(Obj,chNo,value);
        out = getSymmetry(Obj,chNo)
    end
    
    
    %% Frequency-Group ("Commands for channels", full)
    % FREQ           -> setFreq(Obj,1,value)
    % FREQ?          -> getFreq(Obj,1)
    % FREQ:CH2       -> getFreq(Obj,1)
    % FREQ:CH2?      -> setFreq(Obj,2,value)
    % FREQ:STAR      -> setFreq(Obj,1,value,'start')
    % FREQ:STAR?     -> getFreq(Obj,1,'start')
    % FREQ:STOP      -> setFreq(Obj,1,value,'stop')
    % FREQ:STOP?     -> getFreq(Obj,1,'stop')
    % FREQ:CENT      -> setFreq(Obj,1,value,'center')
    % FREQ:CENT?     -> getFreq(Obj,1,'center')
    % FREQ:SPAN      -> setFreq(Obj,1,value,'span')
    % FREQ:SPAN?     -> getFreq(Obj,1,'span')
    methods
        setFreq(Obj,chNo,value,keyword)
        out =  getFreq(Obj,chNo,keyword);
    end
    
    
    %% Voltage-Group ("Commands for channels", full)
    % VOLT           -> setVoltage(Obj,chNo,value)
    % VOLT?          -> getVoltage(Obj,chNo)
    % VOLT:HIGH      -> setVoltage(Obj,chNo,value,'high')
    % VOLT:HIGH?     -> getVoltage(Obj,chNo,'high')
    % VOLT:LOW       -> setVoltage(Obj,chNo,value,'low')
    % VOLT:LOW?      -> getVoltage(Obj,chNo,'low')
    % VOLT:OFFS      -> setVoltage(Obj,chNo,value,'offset')
    % VOLT:OFFS?     -> getVoltage(Obj,chNo,'offset')
    % VOLT:UNIT      -> setVoltage(Obj,chNo,value,'unit')
    % VOLT:UNIT?     -> getVoltage(Obj,chNo,'unit')
    %
    % same for CH2
    methods
        setVoltage(Obj,chNo,value,keyword)
        out =  getVoltage(Obj,chNo,keyword)
    end
    
    
    %% Output-Group
    % OUTP            -> setOutput
    % OUTP?           -> getOutput
    % OUTP:LOAD
    % OUTP:LOAD?
    % OUTP:POL
    % OUTP:POL?
    % OUTP:SYNC       -> setSync
    % OUTP:SYNC?      -> getSync
    % OUTP:TRIG:SLOP
    % OUTP:TRIG:SLOP?
    % OUTP:TRIG
    % OUTP:TRIG?
    % OUTP:CH2        -> setOutput
    % OUTP:CH2?       -> getOutput
    % OUTP:LOAD:CH2
    % OUTP:LOAD:CH2?
    % OUTP:POL:CH2
    % OUTP:POL:CH2?
    methods
        setOutput(obj,ch,inp);
        out = getOutput(obj,ch);
        setSync(obj,inp);
        out = getSync(obj);
    end
    
    
    %% Data-Group (full)
    % DATA       not implemented (can be replaced by DATA:DAC)
    % DATA:DAC        -> setData
    % DATA:COPY       -> copyData
    % DATA:DEL        -> deleteData
    % DATA:CAT?       -> getCatalog(obj,'all')
    % DATA:RENAME     -> renameData
    % DATA:NVOL:CAT?  -> getCatalog(obj,'nvol')
    % DATA:NVOL:FREE? -> getFreeMem, only internal use
    % DATA:ATTR:POIN? only internal use
    % DATA:LOAD       -> getData
    %
    % There's an error between the data which has been sent and that
    % which has been read back:
    % setData(Obj,wav1);
    % wav2 = getData(Obj,'VOLATILE'),
    % wav1(2:end-1) == wav2(1:end-2)
    methods
        setData(obj,waveformdata);
        copyData(Obj,waveformname);
        deleteData(Obj,waveformname);
        renameData(Obj,oldname,newname);
        Out = getCatalog(Visa,varargin);
        out = getFreeMem(Visa);
        out = getData(obj,waveformname);
    end
    
    
    %% Memory-Group: not implemented
    % Remote control can store different configurations of instrument on
    % PC - so there is no requirement for this group
    
    
    %% Phase-Group (full)
    % PHAS            -> setPhase
    % PHAS?           -> getPhase
    % PHAS:CH2        -> setPhase
    % PHAS:CH2?       -> getPhase
    % PHAS:ALIGN      -> alignPhase
    methods
        setPhase(Obj,chNo,value);
        out = getPhase(Obj,chNo);
        alignPhase(Obj);
    end
    
    
    %% System-Group (full)
    % SYST:ERR?       -> getError
    % SYST:VERS       -> getVersion
    % SYST:BEEP:STAT  -> setBeepstate
    % SYST:BEEP:STAT? -> getBeepstate
    % SYST:LOC        -> setLock(obj,'local')
    % SYST:RWL        -> setLock(obj,'lock')
    % SYST:REM        -> setLock(obj,'remote')
    % SYST:CLKSRC not implemented (I don't own a 10MHz TCXO)
    % SYST:LANG   not implemented (I can't read chinese)
    methods
        Out = getError(obj);
        out = getVersion(obj);
        out = getBeepstate(obj);
        setBeepstate(obj,inp);
        setLock(obj);
    end
    
    
    %% Display-Group: not implemented
    % I do not see any reason why one should remote-control an instrument's
    % display
    % DISP?       not supported by instrument (appears in older versions
    %             of DG1022 programming manual)
    
    
    %% Coupling-Group: not implemented (yet)
    %  not implemented yet due to no requirements
    
    
    %% Counter-Group
    % This group is new in Programming Guide from 2014
    % not implemented yet
    
    
    %% Other functions
    % without SCPI pendant
    methods(Static)
        % Read and write waveform files (e.g. for USB-Stick)
        writerdf(filename,data);
        out = readrdf(filename);
        
        % Generate waveform data for testing purpose
        fn = testwaveform(nSample,no);
        
        % Normalize waveform data for use as DAC-data
        fn = normalizewaveform(f);
        
        % Instrument Settings and Limitation DG1022
        S = instrumentProperties(obj,varargin);
        
        % Helper functions
        isOk = hlp_checkinstallation();
        isvalid = hlp_isvalidwaveformname(waveformname);
    end
    
    methods (Hidden=true)
        % helper functions
        Visa = help_getVisa(obj);
    end
    
    
    
end