function out = getData(Obj,waveformname)
% DG1022: waveform data of instrument
%
% getData(obj,waveformname)
%
% obj: Instrument/Visa object
% waveformname: Name of Waveform in NVOL memory
%
% Execution of this function may take several seconds, since there is much
% instrument communication involved.
%
% 8.11.2014 Nicolas (Check www.dl1dow.de for a more recent version)

% This implementation needs only a valid Visa Object. No other Visa
% commands than 'fprintf' and 'fgets' are used.
% Built-in-waveforms cannot be dumped to PC this way.
%
% Implementation was a bit tricky because of the poor documentation of the
% Rigol DG1022. If anyone find better documentation than the programming
% manual, please write me an email (adress see above)!
%
% There's a small error: First data and last point are always missing!

Visa = hlp_getVisa(Obj);


%% Pre-Checks

% Operation will be hard enough, so let's ensure that there is no error
% pending
clrdevice(Visa);
E = getError(Obj);
if ~(numel(E)==1)
    disp(E)
    error('Instrument Error')
end


% Check if desired waveform name is available in instrument
if strcmpi(waveformname,'VOLATILE')
    Available = getCatalog(Obj,'all');
else
    Available = getCatalog(Obj,'nvol');
end
if ~any(strcmpi(Available,waveformname))
    error(['Waveform with name ''%s'' is not available '...
        'in instrument memory'],waveformname);
end

% Just to be sure
E = getError(Obj);
if ~(numel(E)==1)
    disp(E);
    error('Instrument Error')
end


%% Request number of samples
fprintf(Visa,'DATA:ATTR:POIN? %s',waveformname);
pause(0.02);
str = fgets(Visa);
nSample = str2double(str);


%% Get waveform data header
% First request with 'DATA:LOAD <waveformname>
% return value from Instrument is size description
%
% This query seems to be rather hard for the instrument - it timeouts
% often - but when is works, it works fast. So it's worth to give multiple
% tries (useually 2 are needed!)
oldtimeout = get(Visa,'Timeout');
set(Visa,'Timeout',0.5);
nTries = 4;
for i = 1:nTries
    
    % Try to query DATA header
    fprintf(Visa,'DATA:LOAD %s',waveformname);
    pause(0.01);
    [tline,count,~] = fgets(Visa);
   
    % Extract data from return values
    if count==0
        % Probably a VISA timeout
        continue;
    end
    
    C = regexpi(tline,',','split');
    if strcmpi(C{1},'ArbData') && strcmpi(C{2},'HEADER')
        % If the line "ArbData,HEADER,<nSample>, all is OK. If not,
        % give if another try.
        break;
    end
    
end
assert(nSample == str2double(C{3}),...
    'Expected n=%i, got n=%i',nSample,str2double(C));

set(Visa,'Timeout',oldtimeout);

if (i>2) 
    warning('More than 2 tries needed!');
end


%% Get waveform data
% Second request with 'DATA:LOAD <waveformname>
% return value from Instrument is raw DAC data
pause(0.1);
fprintf(Visa,'DATA:LOAD %s',waveformname);
pause(0.1)
[A,count,~] = fread(Visa,nSample,'uint16');

if ~(count==nSample)
    warning('Could not read full data set');
end


out = mod(A,2^14);



