function setDutycycle(Obj,chNo,value)
% DG1022: Set duty cycle of square waveform
%
% setDutycycle(Obj,chNo,value)
%
% value is between 0.2 and 0.8
%
% Limits: 
%  0.2 to 0.8  below 3MHz
%  0.4 to 0.6  3MHz to 4Mhz
%  0.5 only    4Mhz to 5Mhz
%
% 15.11.2014 Nicolas (Check www.dl1dow.de for a more recent version)

% This implementation needs only a valid Visa Object. No other Visa
% commands than 'fprintf' and 'fgets' are used.


Visa = hlp_getVisa(Obj);

% Operation will be hard enough, so let's ensure that there is no error
% pending
clrdevice(Visa);
E = getError(Obj);
if ~(numel(E)==1)
    disp(E)
    error('Instrument Error')
end


%% Interprete channel choice
% Get chSuffix string for scpi command string, get chPrefix for interpretation
% of return argument for choosing the instrument's channel
% For channel 1, no chSuffix is allowed,
% for channel 2, chSuffix is ':CH2'
switch(chNo)
    case 1
        chSuffix = '';
    case 2
        chSuffix = ':CH2';
    otherwise
        error('invalid channelNo');
end


%% Assert this is only with waveform 'Squ'
func = getFunc(Obj,chNo);
if ~strcmpi(func,'Squ')
    error('setDutycycle only makes sense if waveform is ''Squ'' ');
end


%% Assert value is in range
% valid rande is frequency dependent
freq = getFreq(Obj,chNo);
if freq < 3e6
    vMin = 0.2;
    vMax = 0.8;
elseif freq > 5e6
    vMin = 0.5;
    vMax = 0.5;
else
    vMin = 0.4;
    vMax = 0.6;
end

if value<vMin || value>vMax
    error('For a frequency of %f, input must be between %f and %f',...
        freq,vMin,vMax);
end


%% Send command to instrument
percent = value*100;
qstr = sprintf('FUNC:SQU:DCYC%s %s',chSuffix,num2str(percent));
fprintf(Visa,qstr);
pause(0.01);


%% Verify value
v = getDutycycle(Obj,chNo);
if abs(v-value) > 1e-6
    error('Unexpected duty cycle value: Expected %d, got %d',...
        value,v);
end


