function setFreq(Obj,chNo,value,keyword)
% DG1022: Get frequency of channel
%
% out =  getFreq(Obj,chNo)
% out =  getFreq(Obj,1,'start')
% out =  getFreq(Obj,1,'stop')
% out =  getFreq(Obj,1,'center')
% out =  getFreq(Obj,1,'span')
%
% When setting 'center' and 'span' in sweep mode, no warning will occur if
% the instrument's maximum value is exceeded.
%
% 15.11.2014 Nicolas (Check www.dl1dow.de for a more recent version)

% This implementation needs only a valid Visa Object. No other Visa
% commands than 'fprintf' and 'fgets' are used.


Visa = hlp_getVisa(Obj);
Config = DG1022.instrumentProperties;


% Operation will be hard enough, so let's ensure that there is no error
% pending
clrdevice(Visa);
E = getError(Obj);
if ~(numel(E)==1)
    disp(E)
    error('Instrument Error')
end


%% Interprete channel choice
% Get chSuffix string for scpi command string, get chPrefix for interpretation
% of return argument for choosing the instrument's channel
% For channel 1, no chSuffix is allowed,
% for channel 2, chSuffix is ':CH2'
switch(chNo)
    case 1
        % suff
        chSuffix = '';
    case 2
        chSuffix = ':CH2';
    otherwise
        error('invalid channelNo');
end


%% Compare value to instrument's current waveform
% Check if setting freq makes sense and if value is inside the allowed
% range
waveform = getFunc(Obj,chNo)
switch waveform
    case {'Sin','Squ','Ramp','Puls','Arb'}
        % Sweep also reads as 'Sin', so limiting will work here, too
        % Only if 1/2*span+center exceed limit, this will not be warned
        min = Config.(waveform).FreqMin;
        max = Config.(waveform).FreqMax;
    case 'Noise'
        error('Setting frequency in mode ''Noise'' makes no sense');
    otherwise
        % probably an arbitrary waveform
        S = getCatalog(Obj,'all');
        if any(strcmpi(waveform,S))
            % This is an arbitrary waveform
            min = Config.Arb.FreqMin;
            max = Config.Arb.FreqMax;
        else
            % This is something unexpected
            error('unexpected case');
        end
end

if value<min || value > max
    error('In %s waveform, frequency must be between %i and %i',...
        waveform,min,max)
end


%% Interprete keyword
if nargin == 3
    % no keyword
    keyword = 'norm';
else
    % Setting center, span, start or stop only makes sense in Sweep mode
    if ~any(strcmpi(waveform,{'Sweep'}))
        warning('Setting ''%s'' makes no sense in instrument mode ''%s'' ',...
            keyword,waveform);
    end
end

switch lower(keyword)
    case 'norm'
        modeSuffix = '';
    case {'star','start'}
        modeSuffix = ':STAR';
    case {'stop'}
        modeSuffix = ':STOP';
    case {'span'}
        modeSuffix = ':SPAN';
    case {'center','cent'}
        modeSuffix = ':CENT';
    otherwise
        error('unknown keyword: ''%s'' ',keyword);
end

% Most modes are only allowed by Channel 1:
if chNo == 2 && ~strcmpi(modeSuffix,'')
    error('Channel 2 does not allow this mode: ''%s'' ',keyword);
end





%% Send command to instrument
qstr = sprintf('FREQ%s%s %i',modeSuffix,chSuffix,value);
fprintf(Visa,qstr);
pause(0.01);


%% Verify
fret = getFreq(Obj,chNo,keyword);
if fret~=value
    error('Could not verify frequency: Expected %i, got %i',...
        value,fret);
end


